from vimba import *
from Lights import dim_up, dim_down
from json_read_write import get_value_from_section
import time
from datetime import datetime
import os
import cv2
from OLED_panel import display_text
from log import log_schreiben
from GPIO_Setup import *
from sensor_data import read_sensor_data
from error_handling import error_message

try:
    project_name = get_value_from_section("/home/Ento/LepmonOS/Lepmon_config.json","general","project_name")
    sensor_id = get_value_from_section("/home/Ento/LepmonOS/Lepmon_config.json","general","serielnumber")
    province = get_value_from_section("/home/Ento/LepmonOS/Lepmon_config.json","locality","province")
    city_code = get_value_from_section("/home/Ento/LepmonOS/Lepmon_config.json","locality","city")
except Exception as e:
    error_message(11,e)

         

def get_frame(Exposure):
    with Vimba.get_instance() as vimba:
        cams = vimba.get_all_cameras()

        try:
            with cams[0] as cam:
                formats = cam.get_pixel_formats()

                cam.set_pixel_format(PixelFormat.Bgr8)

                settings_file = '/home/Ento/LepmonOS/Kamera_Einstellungen.xml'.format(cam.get_id())
                cam.load_settings(settings_file, PersistType.All)

                
                exposure_time = cam.ExposureTime
                exposure_time.set(Exposure*1000)
                print(f"Exposure geändert:{(exposure_time.get()/1000):.0f}")
                
                frame = cam.get_frame(timeout_ms=5000).as_opencv_image()
                
        except Exception as e:
                    error_message(1,e)
                    time.sleep(3)
                    print(f"Fehler beim Abrufen des Frames: {e}")
                    frame = None
                    print(f"'Frame: {frame}'")
            
        return frame
        
#####################




def snap_image(file_extension,mode,Kamera_Fehlerserie,Exposure):
    """
    nimmt ein Bild auf

    :param file_extension: Dateierweiterung
    :param mode: "display" für lokale ausgabe oder "log" für speichern in der schleife
    """
    ordnerpfad = get_value_from_section("/home/Ento/LepmonOS/Lepmon_config.json","general","current_folder")
    now = datetime.now()
    code = f"{project_name}{sensor_id}_{province}_{city_code}_{now.strftime('%Y')}-{now.strftime('%m')}-{now.strftime('%d')}_T_{now.strftime('%H%M')}"
    image_file = f"{code}.{file_extension}"
    dateipfad = os.path.join(ordnerpfad, image_file)

    turn_on_led("Kamera")
    print("Kamera Strom aktiviert")

    
    
    if mode == "display":
        display_text("Kamera Test","Kamera wird", "eingeschaltet",5)
        display_text("Dimme LED","hoch","",1)
        ordnerpfad = "/home/Ento/LepmonOS/"
        power_on = 0
        
    if mode == "log":
        time.sleep(5)    

    dim_up()
    print("dimme LED hoch")
    frame = get_frame(Exposure)

    if frame is not None:
        Kamera_Fehlerserie = 0

    if frame is None and mode == "display":
        error_message(1,"Fehler beim Abrufen des Frames",3)

    if frame is None and mode == "log": 
        Status_Kamera = 0   
        Kamera_Fehlerserie += 1

    if mode == "log":
        power_on = read_sensor_data("LED-an","now")
        power_on = power_on["power"]
        
    dim_down()

    print("dimme LED runter")
    if mode == "display":
        display_text("Dimme LED","herunter","",1)

    try:
        cv2.imwrite(dateipfad, frame)
        print(f"Bild gespeichert: {dateipfad}")
        Status_Kamera = 1
        if mode == "display":
            display_text("Bild","gespeichert","",2)
            os.remove(dateipfad)
            print(f"Bild vom Speicher gelöscht: {dateipfad}")
            log_schreiben("Kamera Zugriff erfolgreich")
        if mode == "log": 
            log_schreiben(f"Bild gespeichert: {dateipfad}")             

    except Exception as e:
        Status_Kamera = 0
        print(f"Kamerafehler:{e}")

    return code, dateipfad, Status_Kamera, power_on, Kamera_Fehlerserie


if __name__ == "__main__":
    print("Starte Kamera...")
    snap_image("jpg","log",0,5)                